@@ -14,8 +14,16 @@ import logging
 import shutil
 from typing import Callable, Optional
 from urllib import request
-import cv2 as cv
-from natsort import natsorted
+try:
+    import cv2 as cv
+except ImportError as e:
+    cv = None
+
+try:
+    import cv2 as cv
+except ImportError as e:
+    cv = None
+
 
 __all__ = ["PathManager", "get_cache_dir", "file_lock"]
 
@@ -676,6 +684,8 @@ class ImageSourceIter(SourceIter):
             return NotImplementedError
 
     def _index_sources(self):
+        from natsort import natsorted
+
         if str(self.src).isdigit():
             self.webcam_mode = True
             self.video_mode = True
@@ -710,6 +720,18 @@ class ImageSourceIter(SourceIter):
                     # clean up remove saved file.
                     os.remove(self.save_f)
 
+    def waitKey(self):
+        if self.video_mode:
+            cv.waitKey(1)
+        else:
+            cv.waitKey(0)
+
+
+def next_item(iter):
+    itm = next(iter)
+    if isinstance(itm, str):
+        itm = cv.imread(itm)
+    return itm
 
 class ImageSourceIterAsync(SourceIter):
     """
@@ -719,7 +741,8 @@ class ImageSourceIterAsync(SourceIter):
 
     def __init__(self, src, exit_auto=True):
         super(ImageSourceIter, self).__init__(src, exit_auto)
-
+        import cv2 as cv
+        
         self._index_sources()
         self.is_written = False
         self.save_f = None
@@ -794,6 +817,8 @@ class ImageSourceIterAsync(SourceIter):
             return NotImplementedError
 
     def _index_sources(self):
+        from natsort import natsorted
+
         if str(self.src).isdigit():
             self.webcam_mode = True
             self.video_mode = True
