@@ -6,25 +6,25 @@ import math
 from alfred.vis.image.common import colors
 
 BUILTIN_JOINTS_17_COCO = [
-    [1, 2],
-    [1, 5],
-    [2, 3],
-    [3, 4],
-    [5, 6],
-    [6, 7],
-    [1, 8],
-    [8, 9],
-    [9, 10],
-    [1, 11],
-    [11, 12],
+    [16, 14],
+    [14, 12],
+    [17, 15],
+    [15, 13],
     [12, 13],
-    [1, 0],
-    [0, 14],
-    [14, 16],
-    [0, 15],
-    [15, 17],
-    [2, 16],
-    [5, 17],
+    [6, 12],
+    [7, 13],
+    [6, 7],
+    [6, 8],
+    [7, 9],
+    [8, 10],
+    [9, 11],
+    [2, 3],
+    [1, 2],
+    [1, 3],
+    [2, 4],
+    [3, 5],
+    [4, 6],
+    [5, 7],
 ]
 
 BUILTIN_JOINTS_26_HALPE = [
@@ -58,25 +58,146 @@ BUILTIN_JOINT_28_COMBINED = []
 
 
 def vis_pose_by_joints(
-    img, poses, joints, color=colors(np.random.randint(30)), point_color=[252, 219, 3], point_size=6
+    img,
+    poses,
+    joints,
+    color=colors(np.random.randint(30)),
+    point_color=None,
+    point_size=6,
+    line_width=2,
+    face_connect=False,
+    joints_reset_zero=True,
+    aa=False,
 ):
-    N, num_kps, _ = poses.shape
+    if len(poses.shape) == 2:
+        poses = np.array([poses])
+        N, num_kps, _ = poses.shape
+    else:
+        N, num_kps, _ = poses.shape
+    assert isinstance(joints, np.ndarray), "joints must be numpy array!"
+    if len(joints.shape) == 3:
+        joints = joints[0]
+
+    if point_color is None:
+        point_color = color
+
     for pose in poses:
         # for part_id in range(num_kps - 1):
         for part_id in range(len(joints)):
             # if num_kps is 18, then 0 ... 17
-            kpt_a_id = joints[part_id][0]
-            global_kpt_a_id = pose[kpt_a_id, 0]
-            if global_kpt_a_id != -1:
-                x_a, y_a = pose[kpt_a_id]
+            kpt_a_id = (
+                joints[part_id][0] - 1 if joints_reset_zero else joints[part_id][0]
+            )
+            global_kpt_a = pose[kpt_a_id].tolist()
+            if len(global_kpt_a) == 2:
+                x_a, y_a = global_kpt_a
+            else:
+                x_a, y_a, _ = global_kpt_a
+            if x_a > 0 and y_a > 0:
                 cv2.circle(img, (int(x_a), int(y_a)), 3, color, -1, cv2.LINE_AA)
-            kpt_b_id = joints[part_id][1]
-            global_kpt_b_id = pose[kpt_b_id, 0]
-            if global_kpt_b_id != -1:
-                x_b, y_b = pose[kpt_b_id]
-                cv2.circle(img, (int(x_b), int(y_b)), point_size, point_color, -1, cv2.LINE_AA)
-            if global_kpt_a_id != -1 and global_kpt_b_id != -1:
-                cv2.line(img, (int(x_a), int(y_a)), (int(x_b), int(y_b)), color, 2, cv2.LINE_AA)
+
+            kpt_b_id = (
+                joints[part_id][1] - 1 if joints_reset_zero else joints[part_id][1]
+            )
+            global_kpt_b = pose[kpt_b_id].tolist()
+            if len(global_kpt_b) == 2:
+                x_b, y_b = global_kpt_b
+            else:
+                x_b, y_b, _ = global_kpt_b
+            if x_b > 0 and y_b > 0:
+                if (
+                    kpt_a_id not in [0, 1, 2, 3, 4]
+                    and kpt_b_id not in [0, 1, 2, 3, 4]
+                    and not face_connect
+                ):
+                    cv2.circle(
+                        img,
+                        (int(x_b), int(y_b)),
+                        point_size + 1,
+                        point_color,
+                        -1,
+                        cv2.LINE_AA if aa else cv2.LINE_8,
+                    )
+                else:
+                    cv2.circle(
+                        img,
+                        (int(x_b), int(y_b)),
+                        point_size,
+                        point_color,
+                        -1,
+                        cv2.LINE_AA if aa else cv2.LINE_8,
+                    )
+            if x_a > 0 and y_a > 0 and x_b > 0 and y_b > 0:
+                if face_connect:
+                    cv2.line(
+                        img,
+                        (int(x_a), int(y_a)),
+                        (int(x_b), int(y_b)),
+                        color,
+                        line_width,
+                        cv2.LINE_AA if aa else cv2.LINE_8,
+                    )
+                else:
+                    if kpt_a_id not in [0, 1, 2, 3, 4] and kpt_b_id not in [
+                        0,
+                        1,
+                        2,
+                        3,
+                        4,
+                    ]:
+                        cv2.line(
+                            img,
+                            (int(x_a), int(y_a)),
+                            (int(x_b), int(y_b)),
+                            color,
+                            line_width,
+                            cv2.LINE_AA if aa else cv2.LINE_8,
+                        )
+    return img
+
+
+def vis_pose_coco_17(
+    img,
+    poses,
+    color=colors(np.random.randint(30)),
+    point_color=[252, 219, 3],
+    point_size=6,
+):
+    joints = BUILTIN_JOINTS_17_COCO
+    assert isinstance(poses, np.ndarray), "poses must be numpy array!"
+    N, num_kps, last_dim = poses.shape
+    for pose in poses:
+        # for part_id in range(num_kps - 1):
+        for part_id in range(len(joints)):
+            # if num_kps is 18, then 0 ... 17
+            kpt_a_id = joints[part_id][0] - 1
+            global_kpt_a = pose[kpt_a_id].tolist()
+            x_a, y_a, _ = global_kpt_a
+            if x_a >= 0 and y_a >= 0:
+                cv2.circle(img, (int(x_a), int(y_a)), 3, color, -1, cv2.LINE_AA)
+
+            kpt_b_id = joints[part_id][1] - 1
+            global_kpt_b = pose[kpt_b_id].tolist()
+            x_b, y_b, _ = global_kpt_b
+            if x_b >= 0 and y_b >= 0:
+                x_b, y_b, _ = global_kpt_b
+                cv2.circle(
+                    img, (int(x_b), int(y_b)), point_size, point_color, -1, cv2.LINE_AA
+                )
+            if (
+                global_kpt_a[0] >= 0
+                and global_kpt_a[1] >= 0
+                and global_kpt_b[0] >= 0
+                and global_kpt_b[1] >= 0
+            ):
+                cv2.line(
+                    img,
+                    (int(x_a), int(y_a)),
+                    (int(x_b), int(y_b)),
+                    color,
+                    2,
+                    cv2.LINE_AA,
+                )
     return img
 
 
@@ -193,7 +314,7 @@ def imshow_keypoints(
                                 (int(x_coord), int(y_coord)),
                                 radius,
                                 (int(r), int(g), int(b)),
-                                -1,
+                                2,
                                 cv2.LINE_AA,
                             )
                             transparency = max(0, min(1, kpt_score))
@@ -211,8 +332,9 @@ def imshow_keypoints(
                                 img,
                                 (int(x_coord), int(y_coord)),
                                 radius,
-                                (int(r), int(g), int(b)),
-                                -1,
+                                # (int(r), int(g), int(b)),
+                                (255, 255, 255),
+                                2,
                                 cv2.LINE_AA,
                             )
                 else:
@@ -238,8 +360,9 @@ def imshow_keypoints(
                             img,
                             (int(x_coord), int(y_coord)),
                             radius,
-                            (int(r), int(g), int(b)),
-                            -1,
+                            # (int(r), int(g), int(b)),
+                            (255, 255, 255),
+                            2,
                             cv2.LINE_AA,
                         )
 
@@ -357,3 +480,216 @@ def imshow_keypoints(
                             )
 
     return img
+
+
+######## HALPE
+l_pair = [
+    (0, 1),
+    (0, 2),
+    (1, 3),
+    (2, 4),  # Head
+    (5, 18),
+    (6, 18),
+    (5, 7),
+    (7, 9),
+    (6, 8),
+    (8, 10),  # Body
+    (17, 18),
+    (18, 19),
+    (19, 11),
+    (19, 12),
+    (11, 13),
+    (12, 14),
+    (13, 15),
+    (14, 16),
+    (20, 24),
+    (21, 25),
+    (23, 25),
+    (22, 24),
+    (15, 24),
+    (16, 25),  # Foot
+    (26, 27),
+    (27, 28),
+    (28, 29),
+    (29, 30),
+    (30, 31),
+    (31, 32),
+    (32, 33),
+    (33, 34),
+    (34, 35),
+    (35, 36),
+    (36, 37),
+    (37, 38),  # Face
+    (38, 39),
+    (39, 40),
+    (40, 41),
+    (41, 42),
+    (43, 44),
+    (44, 45),
+    (45, 46),
+    (46, 47),
+    (48, 49),
+    (49, 50),
+    (50, 51),
+    (51, 52),  # Face
+    (53, 54),
+    (54, 55),
+    (55, 56),
+    (57, 58),
+    (58, 59),
+    (59, 60),
+    (60, 61),
+    (62, 63),
+    (63, 64),
+    (64, 65),
+    (65, 66),
+    (66, 67),  # Face
+    (68, 69),
+    (69, 70),
+    (70, 71),
+    (71, 72),
+    (72, 73),
+    (74, 75),
+    (75, 76),
+    (76, 77),
+    (77, 78),
+    (78, 79),
+    (79, 80),
+    (80, 81),  # Face
+    (81, 82),
+    (82, 83),
+    (83, 84),
+    (84, 85),
+    (85, 86),
+    (86, 87),
+    (87, 88),
+    (88, 89),
+    (89, 90),
+    (90, 91),
+    (91, 92),
+    (92, 93),  # Face
+    (94, 95),
+    (95, 96),
+    (96, 97),
+    (97, 98),
+    (94, 99),
+    (99, 100),
+    (100, 101),
+    (101, 102),
+    (94, 103),
+    (103, 104),
+    (104, 105),  # LeftHand
+    (105, 106),
+    (94, 107),
+    (107, 108),
+    (108, 109),
+    (109, 110),
+    (94, 111),
+    (111, 112),
+    (112, 113),
+    (113, 114),  # LeftHand
+    (115, 116),
+    (116, 117),
+    (117, 118),
+    (118, 119),
+    (115, 120),
+    (120, 121),
+    (121, 122),
+    (122, 123),
+    (115, 124),
+    (124, 125),  # RightHand
+    (125, 126),
+    (126, 127),
+    (115, 128),
+    (128, 129),
+    (129, 130),
+    (130, 131),
+    (115, 132),
+    (132, 133),
+    (133, 134),
+    (134, 135),  # RightHand
+]
+p_color = [
+    (0, 255, 255),
+    (0, 191, 255),
+    (0, 255, 102),
+    (0, 77, 255),
+    (0, 255, 0),  # Nose, LEye, REye, LEar, REar
+    (77, 255, 255),
+    (77, 255, 204),
+    (77, 204, 255),
+    (191, 255, 77),
+    (77, 191, 255),
+    (191, 255, 77),  # LShoulder, RShoulder, LElbow, RElbow, LWrist, RWrist
+    (204, 77, 255),
+    (77, 255, 204),
+    (191, 77, 255),
+    (77, 255, 191),
+    (127, 77, 255),
+    (77, 255, 127),  # LHip, RHip, LKnee, Rknee, LAnkle, RAnkle, Neck
+    (77, 255, 255),
+    (0, 255, 255),
+    (77, 204, 255),  # head, neck, shoulder
+    (0, 255, 255),
+    (0, 191, 255),
+    (0, 255, 102),
+    (0, 77, 255),
+    (0, 255, 0),
+    (77, 255, 255),
+]  # foot
+
+line_color = [
+    (0, 215, 255),
+    (0, 255, 204),
+    (0, 134, 255),
+    (0, 255, 50),
+    (0, 255, 102),
+    (77, 255, 222),
+    (77, 196, 255),
+    (77, 135, 255),
+    (191, 255, 77),
+    (77, 255, 77),
+    (77, 191, 255),
+    (204, 77, 255),
+    (77, 222, 255),
+    (255, 156, 127),
+    (0, 127, 255),
+    (255, 127, 77),
+    (0, 77, 255),
+    (255, 77, 36),
+    (0, 77, 255),
+    (0, 77, 255),
+    (0, 77, 255),
+    (0, 77, 255),
+    (255, 156, 127),
+    (255, 156, 127),
+]
+
+
+def vis_halpe(img, kpts, kpts_scores):
+    # n,26,3
+    for b in range(kpts.shape[0]):
+        kp_x = kpts[b][..., 0]
+        kp_y = kpts[b][..., 1]
+        kp_scores = kpts_scores[b]
+        part_line = {}
+        # Draw keypoints
+        for n in range(kp_scores.shape[0]):
+            if kp_scores[n] <= 0.6:
+                continue
+            cor_x, cor_y = int(kp_x[n]), int(kp_y[n])
+            part_line[n] = (int(cor_x), int(cor_y))
+            if n < len(p_color):
+                cv2.circle(img, (int(cor_x), int(cor_y)), 3, p_color[n], -1)
+            else:
+                cv2.circle(img, (int(cor_x), int(cor_y)), 3, (25, 122, 34), 2)
+        # Draw limbs
+        for i, (start_p, end_p) in enumerate(l_pair):
+            if start_p in part_line and end_p in part_line:
+                start_xy = part_line[start_p]
+                end_xy = part_line[end_p]
+                if i < len(line_color):
+                    cv2.line(img, start_xy, end_xy, line_color[i], 3)
+                else:
+                    cv2.line(img, start_xy, end_xy, (25, 55, 155), 2)
+    return img
